﻿local addonName = ...

local ScriptErrorsFrame = ScriptErrorsFrame
local ScrollFrame = ScriptErrorsFrame.ScrollFrame
local EditBox, ScrollBar = ScrollFrame.Text, ScrollFrame.ScrollBar

--[[----------------------------------------------------------------------------
Style adjustments
------------------------------------------------------------------------------]]
ScriptErrorsFrame.Reload:Hide()

local function ReplaceButton(parent, index)
	local oldButton = parent[index]
	if not oldButton then return end
	local newButton = CreateFrame('Button', nil, parent, 'UIPanelButtonTemplate')
	newButton:SetID(oldButton:GetID())
	newButton:SetScript('OnClick', oldButton:GetScript('OnClick'))
	oldButton:SetScript('OnClick', nil)
	oldButton:Hide()
	parent[index] = newButton
end
	
ReplaceButton(ScriptErrorsFrame, 'PreviousError')
ReplaceButton(ScriptErrorsFrame, 'NextError')

--[[----------------------------------------------------------------------------
Reposition and relabel buttons and index
------------------------------------------------------------------------------]]
local function AdjustButton(button, label, ...)
	if not button then return end
	button:SetSize(32, 20)
	button:SetText(label)
	button:GetFontString():SetJustifyH('CENTER')
	button:ClearAllPoints()
	button:SetPoint(...)
	return button
end

local indexLabel = ScriptErrorsFrame.IndexLabel
indexLabel:SetJustifyH('CENTER')
indexLabel:SetText("8888 / 8888")
indexLabel:SetWidth(0)
indexLabel:SetWidth(indexLabel:GetStringWidth())
indexLabel:ClearAllPoints()
indexLabel:SetPoint('BOTTOM', 0, 8)

local close = AdjustButton(ScriptErrorsFrame.Close, "><", 'BOTTOMLEFT', ScriptErrorsFrame, 7, 8)	-- Don't need two close buttons
close:SetScript('OnClick', function(self)
	local index = tonumber(ScriptErrorsFrame.index)
	if not index or index < 1 or index > #ScriptErrorsFrame.order then return end
	local messageKey = tremove(ScriptErrorsFrame.order, index)
	local count = tremove(ScriptErrorsFrame.count, index)
	tremove(ScriptErrorsFrame.messages, index)
	tremove(ScriptErrorsFrame.times, index)
	ScriptErrorsFrame.seen[messageKey] = nil
	tremove(ScriptErrorsFrame.locals, index)
	tremove(ScriptErrorsFrame.warnType, index)
	if ScriptErrorsFrame.messageCount < ScriptErrorsFrame.messageLimit then
		ScriptErrorsFrame.messageCount = ScriptErrorsFrame.messageCount - count
	end
	ScriptErrorsFrame:Update()
	if #ScriptErrorsFrame.order == 0 then
		ScriptErrorsFrame:Hide()
	end
end)

local previous = AdjustButton(ScriptErrorsFrame.PreviousError, "<", 'BOTTOMRIGHT', indexLabel, 'BOTTOMLEFT', -2, 0)

local next = AdjustButton(ScriptErrorsFrame.NextError, ">", 'BOTTOMLEFT', indexLabel, 'BOTTOMRIGHT', 0, 0)

local first = AdjustButton(CreateFrame('Button', nil, ScriptErrorsFrame, 'UIPanelButtonTemplate'), "<<", 'BOTTOMRIGHT', previous, 'BOTTOMLEFT')
first:SetScript('OnClick', function(self)
	if ScriptErrorsFrame.index ~= 1 and #ScriptErrorsFrame.order > 0 then
		ScriptErrorsFrame.index = 1
		ScriptErrorsFrame:Update()
	end
end)
ScriptErrorsFrame.First = first

local last = AdjustButton(CreateFrame('Button', nil, ScriptErrorsFrame, 'UIPanelButtonTemplate'), ">>", 'BOTTOMLEFT', next, 'BOTTOMRIGHT')
last:SetScript('OnClick', function(self)
	if ScriptErrorsFrame.index ~= #ScriptErrorsFrame.order then
		ScriptErrorsFrame.index = #ScriptErrorsFrame.order
		ScriptErrorsFrame:Update()
	end
end)
ScriptErrorsFrame.Last = last

hooksecurefunc(ScriptErrorsFrame, 'UpdateButtons', function()
	local index, numErrors = ScriptErrorsFrame.index, #ScriptErrorsFrame.order
	ScriptErrorsFrame.Close:SetEnabled(numErrors > 0)
	ScriptErrorsFrame.First:SetEnabled(index > 1)
	ScriptErrorsFrame.Last:SetEnabled(index < numErrors)
end)

--[[----------------------------------------------------------------------------
Info button
------------------------------------------------------------------------------]]
local info = CreateFrame('Frame', nil, ScriptErrorsFrame)
info:SetFrameLevel(ScriptErrorsFrame.DragArea:GetFrameLevel() + 1)
info:SetPoint('RIGHT', ScriptErrorsFrameClose, 'LEFT', 6, 0)
info:SetHitRectInsets(6, 6, 7, 7)
info:SetSize(24, 24)

local icon = info:CreateTexture()
icon:SetTexture([[Interface\Common\help-i]])
icon:SetVertexColor(0.85, 0.85, 0.85, 1)
icon:SetAllPoints()

info:SetScript('OnEnter', function(self)
	local version, build, date, toc = GetBuildInfo()
	GameTooltip:SetOwner(self, 'ANCHOR_NONE')
	GameTooltip:SetPoint('TOPRIGHT', self, 'LEFT', 10, -2)
	GameTooltip:AddLine("GetBuildInfo()")
	GameTooltip:AddDoubleLine("Version", version, 1, 1, 1, 1, 1, 1)
	GameTooltip:AddDoubleLine("Build", build, 1, 1, 1, 1, 1, 1)
	GameTooltip:AddDoubleLine("Date", date, 1, 1, 1, 1, 1, 1)
	GameTooltip:AddDoubleLine("TOC", toc, 1, 1, 1, 1, 1, 1)
	GameTooltip:Show()
end)

info:SetScript('OnLeave', function(self)
	if GameTooltip:GetOwner() == self then
		GameTooltip:Hide()
	end
end)

--[[----------------------------------------------------------------------------
Make it so that resizing won't break appearance
------------------------------------------------------------------------------]]
ScrollBar:ClearAllPoints()
ScrollBar:SetPoint('TOPRIGHT', ScriptErrorsFrameClose, 'BOTTOMRIGHT', -8, -12)
ScrollBar:SetPoint('BOTTOMRIGHT', ScriptErrorsFrame, 'BOTTOMRIGHT', -8, 43)

EditBox:ClearAllPoints()
EditBox:SetPoint('TOPLEFT', ScrollFrame)
EditBox:SetPoint('BOTTOMRIGHT', ScrollFrame)

ScrollFrame:SetScript('OnSizeChanged', function(self, width, height)
	self.Text:SetWidth(width)
end)

ScrollFrame:SetSize(0, 0)
ScrollFrame:ClearAllPoints()
ScrollFrame:SetPoint('TOPLEFT', ScriptErrorsFrame.DragArea, 'BOTTOMLEFT', 2, -3)
ScrollFrame:SetPoint('BOTTOMRIGHT', ScrollBar.ScrollDownButton, 'BOTTOMLEFT', -1, 2)

ScriptErrorsFrameDialogBG:SetTexture([[Interface\FrameGeneral\UI-Background-Marble]])	-- Other one scales badly

ScriptErrorsFrame:SetMinResize(285, 128)

--[[----------------------------------------------------------------------------
Make resizable
------------------------------------------------------------------------------]]
local sizeGrabber = CreateFrame('Button', nil, ScriptErrorsFrame)
sizeGrabber:SetNormalTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-Up]])
sizeGrabber:SetPushedTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-Down]])
sizeGrabber:SetHighlightTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-Highlight]])
sizeGrabber:SetPoint('BOTTOMRIGHT', -3, 6)
sizeGrabber:SetSize(16, 16)

sizeGrabber:SetScript('OnMouseDown', function(self)
	ScriptErrorsFrame:StartSizing('BOTTOMRIGHT')
end)

sizeGrabber:SetScript('OnMouseUp', function(self)
	ScriptErrorsFrame:StopMovingOrSizing()
end)

ScriptErrorsFrame:SetResizable(true)

--[[----------------------------------------------------------------------------
Tweaks
------------------------------------------------------------------------------]]
EditBox.cursorOffset = 0														-- Fix for calls to
EditBox.cursorHeight = 0														-- ScrollingEdit_OnTextChanged

ScriptErrorsFrame:SetScript('OnShow', ScriptErrorsFrame.Update)					-- Avoid unneeded nested function calls
ScrollFrame:SetScript('OnMouseWheel', ScrollFrameTemplate_OnMouseWheel)
ScrollFrame:SetScript('OnScrollRangeChanged', ScrollFrame_OnScrollRangeChanged)
EditBox:SetScript('OnEscapePressed', EditBox.ClearFocus)

EditBox:SetScript('OnUpdate', nil)												-- Replace OnUpdate script with something
EditBox:SetScript('OnCursorChanged', function(self, _, y, _, cursorHeight)		-- less processor heavy
	self, y = ScriptErrorsFrame.ScrollFrame, -y
	local offset = self:GetVerticalScroll()
	if y < offset then
		self:SetVerticalScroll(y)
	else
		y = y + cursorHeight - self:GetHeight()
		if y > offset then
			self:SetVerticalScroll(y)
		end
	end
end)

EditBox:SetScript('OnEditFocusGained', nil)										-- Prevent auto-highlighting

EditBox:SetScript('OnEditFocusLost', function(self)
	self.text = nil																-- Force undo any edits
	ScriptErrorsFrame:Update()
end)

local HighlightText = EditBox.HighlightText
function EditBox:HighlightText()												-- Override auto-highlighting
	HighlightText(self, 0, 0)
	self:ClearFocus()															-- Hack to drop focus on index change
end
HighlightText(EditBox, 0, 0)

if not ScriptErrorsFrame.Title:GetText() then
	ScriptErrorsFrame.Title:SetText(LUA_ERROR)
end

ScriptErrorsFrame:SetFrameStrata('DIALOG')

ScriptErrorsFrame:SetClampRectInsets(6, -3, -4, 6)
