local addonName, addon, L = LibStub('LibOptionsAssist-1.1').FetchData()

local settings = addon.settings

--[[----------------------------------------------------------------------------
Helpers
------------------------------------------------------------------------------]]
local function Get(info)
	return settings[info[#info]]
end

local function Get_CVar(info)
	local cvar = info[#info]
	return GetCVarBool(cvar)
end

local function Get_Event(info)
	local event = info[#info]
	local frame = _G[addon.events[event]]
	return frame:IsEventRegistered(event)
end

local function Set(info, value)
	settings[info[#info]] = value
	addon.ApplySettings()
end

local function Set_CVar(info, value)
	local cvar = info[#info]
	settings[cvar] = value
	SetCVar(cvar, value and '1' or '0')
end

local function Set_Event(info, value)
	local event = info[#info]
	local frame = _G[addon.events[event]]
	settings[event] = value
	frame[value and 'RegisterEvent' or 'UnregisterEvent'](frame, event)
end

--[[----------------------------------------------------------------------------
Options
------------------------------------------------------------------------------]]
local options = {
	type = 'group',
	get = Get,
	set = Set,
	args = {
		closeWithEsc = {
			type = 'toggle',
			order = 1,
			name = L["Escape Closes"],
			desc = L["Allow the escape key to close the error window."],
		},
		reset = {
			type = 'execute',
			order = 2,
			name = L["Reset"],
			desc = L["Load default settings."],
			func = function()
				wipe(settings)
				addon.InitializeSettings()
			end
		}
	}
}

--[[----------------------------------------------------------------------------
cvar Options
------------------------------------------------------------------------------]]
local args, cvars = options.args, addon.cvars

local order = 1
for _ in pairs(args) do
	order = order + 1
end

args.cvarsHeader = {
	type = 'header',
	order = order,
	name = "cvars"
}

order = order + 1
for cvar, frameName in pairs(cvars) do
	args[cvar] = {
		type = 'toggle',
		order = order, width = 'full',
		name = cvar,
		desc = frameName,
		get = Get_CVar,
		set = Set_CVar
	}
end

--[[----------------------------------------------------------------------------
Event Options
------------------------------------------------------------------------------]]
local events = addon.events

order = order + 1
args.eventsHeader = {
	type = 'header',
	order = order,
	name = L["Events"]
}

order = order + 1
for event, frameName in pairs(events) do
	args[event] = {
		type = 'toggle',
		order = order, width = 'full',
		name = event,
		desc = frameName,
		get = Get_Event,
		set = Set_Event
	}
end

--[[----------------------------------------------------------------------------
Initialize
------------------------------------------------------------------------------]]
LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(addonName, options)

addon.config:AssignOptions(addonName)
addon.config:SetDesc(L["These options allow you to change the behavior of %s."]:format(addonName))
addon.config:SetInfo(L["Version: %s"]:format(GetAddOnMetadata(addonName, 'Version')))
